/*
Die Klasse realisiert das Modell im Zeitbereich fr einen PID-Regler.
Es gibt nur eine Eingangsgroesse und eine Zustandsgroesse.
Die allgemeine Formel im Bildbereich lautet:

R(s) = Kp*(1 + 1/(Tn*s) + Tv*s)

Dabei ist Kp der Proportionalfaktor, 
          Tn die Nachregelzeit des Integrators und
          Tv die Verstrkung des differenzierenden Anteils.

Im Zeitbereich ergibt sich das folgende Modell, wenn
y die Ausgangsgroesse ist und u die Eingangsgroesse:

y = Kp*( u + (1/Tn)*Integ(u) + Tv*(du/dt) )

u ist bei der Motor-Drehzahlreglung die Differenz zwischen
Soll-Winkelgeschwindigkeit und Ist-Winkelgeschwindigkeit.

Damit gilt:

u0 = u = wsoll - w

u1 = du/dt = -dw/dt

u2 = integ(u) = integ(wsoll - w) = wsoll*t - phi

y = Kp*( u0 + (1/Tn)*u1 + Tv*u2 )


*/

class AlgebraPID : public Algebra
{
    public:
        double Kp,Tn,Tv;

        AlgebraPID(double Kp, double Tn, double Tv)
        {
            this->Kp = Kp;
            this->Tn = Tn;
            this->Tv = Tv;
        }

        void rechteSeite(double* y,double* u, double t)
        {
            y[0] = Kp*( u[0] + (1/Tn)*u[1] + Tv*u[2] );
        }

        void setParam(double Kp, double Tn, double Tv)
        {
            this->Kp = Kp;
            this->Tn = Tn;
            this->Tv = Tv;
        }
};
