/*
...modifiziert als Zustands-geregeltes System!

Modell eines Gleichstrommotors.
Parameter:
D: viskose Reibung
J: Hauptachsen-Trgheitsmoment des Lufers bzgl. Drehachse
K: Generatorkonstante
R: Ohmscher Widerstand der Luferwicklung
L: Induktivitt der Wicklung

y0: Omega   [rad/s]
y1: Strom i [A]

r1,r2: Einstellungen der Rckfhrung der Zustandsgren auf u

->u -> Modell -> w,i -> u = -r1*w-r2*i -> Modell

Da u/L wirkt, wird auch -r1/L * w und -r2/L * i zurckgefhrt

r1 und r2 werden automatisch bestimmt, knnen spter modifiziert
werden.
r2 zunchst so, dass die EW bei -1 liegen,
dann r1 so, dass die negative Wurzel verschwindet (aperiodischer Grenzfall)

Dem Konstruktor wird noch die Soll-Winkelgeschwindigkeit wsoll
bergeben.

Aus den stationren Gleichungen wird damit auch isoll bestimmt.

Das Entwickeln der Zustandsgleichungen um den stationren Punkt
fhrt zu einem zustzlichen konstanten Vektor vstat.

*/

class ModellMotor : public Modell 
{
    public:
        double D,J,K,R,L,r1,r2,wsoll,isoll,usoll;
        
        double** A;
        double*  vstat;

        ModellMotor(double D, double J, double K, double R, double L, double wsoll)
        {
            this->D = D;
            this->J = J;
            this->K = K;
            this->R = R;
            this->L = L;

            //Bestimmung des Sollstroms aus der
            //Sollwinkelgeschwindigkeit mit Hilfe der
            //stationren Gleichungen (di/dt=0 und dw/dt=0)

            //J*dw/dt = -D*w + k*i und dw/dt=0 =>
            //  isoll = (D/k)*wsoll        

            this->wsoll = wsoll;
            this->isoll = (D/K)*wsoll;
            this->usoll = R*isoll+K*wsoll;
//            this->isoll = (5.0 - K*wsoll)/R;
//            this->isoll = -K*wsoll/R;
//            this->isoll = 0.0;


            //Bestimmung von r1 und r2 wie oben beschrieben:
//r2 so bestimmen, dass Pole bei -10 liegen, also eine sehr schnelle Strecke entsteht.
//Imaginre Anteile zu Null werden lassen, r1 so bestimmen, dass Wurzel zu Null wird.
//            r2 = (20.0 - D/J)*L - R;
//            r1 = -1.0*( (R+r2)*D/(L*J) - 0.25*(D/J+(R+r2)/L)*(D/J+(R+r2)/L) )*L*J/K - K;


//Mit Matlab bestimmte Riccati-Werte:
r1 = 0.8246;
r2 = 0.4714;

            //Bestimmung des stationren Vektors vstat:

            //vorher: dy/dt = Ay + Bu             
            //jetzt:  dy/dt = Ay + vstat -B*R*y

            vstat = new double[yanz()];

//Achtung: r1 und r2 gehen auch in den stationren Teil ein!!!                       
            vstat[0] = (-D/J)*wsoll + (K/J)*isoll;
            vstat[1] = (-K/L-r1/L)*wsoll + (-R/L-r2/L)*isoll;
            vstat[2] = 0.0;

            A = Matrix::erzeugeMatrix(yanz(),yanz());

            A[0][0] = -D/J;            
            A[0][1] =  K/J;            
            A[0][2] =  0.0;            

            A[1][0] = -K/L;            
            A[1][1] = -R/L;            
            A[1][2] =  0.0;            

            A[2][0] =  1.0;            
            A[2][1] =  0.0;            
            A[2][2] =  0.0;            
        }

        void rechteSeite(double* f,double* y, double* u, double t)
        {
            Matrix::malV(f,A,y,yanz(),yanz());

            //u mitberechnen, um es beobachten zu knnen:
            u[0] = (-r1*y[0]-r2*y[1]);

            f[1] += u[0]/L;

            f[0] -= vstat[0];        
            f[1] -= vstat[1];        
            f[2] -= vstat[2];        
        }

        void setR(double* R)
        {
            r1 = R[0];
            r2 = R[1];
        }

        int yanz()
        {
            return 3;
        }
        int uanz()
        {
            return 1;
        }
};
